#include "include/system.h"

#define FLOPPY_PARAMS_ADDRESS = 0x000fefc7  /* Floppy disk parameters reside at this physical address. */
#define FLOPPY_PRIMARY_BASE   = 0x03f0      /* Base address for primary FDC */
#define FLOPPY_SECONDARY_BASE = 0x0370      /* Base address for secondary FDC */

/*
 * Offsets to add to FDC base address to get register:
 */
#define STATUS_REG_A             0x0000 /* PS2 SYSTEMS */
#define STATUS_REG_B             0x0001 /* PS2 SYSTEMS */
#define DIGITAL_OUTPUT_REG       0x0002
#define MAIN_STATUS_REG          0x0004
#define DATA_RATE_SELECT_REG     0x0004 /* PS2 SYSTEMS */
#define DATA_REGISTER            0x0005
#define DIGITAL_INPUT_REG        0x0007 /* AT SYSTEMS */
#define CONFIG_CONTROL_REG       0x0007 /* AT SYSTEMS */
#define PRIMARY_RESULT_STATUS    0x0000
#define SECONDARY_RESULT_STATUS  0x0000

/* 
 * Controller commands:
 */
#define FIX_DRIVE_DATA           0x03
#define CHECK_DRIVE_STATUS       0x04
#define CALIBRATE_DRIVE          0x07
#define CHECK_INTERRUPT_STATUS   0x08
#define FORMAT_TRACK             0x4D
#define READ_SECTOR              0x66
#define READ_DELETE_SECTOR       0xCC
#define READ_SECTOR_ID           0x4A
#define READ_TRACK               0x42
#define SEEK_TRACK               0x0F
#define WRITE_SECTOR             0xC5
#define WRITE_DELETE_SECTOR      0xC9
 
/*
 * Floppy disk parameters structure (found at FLOPPY_PARAMS_ADDRESS 
 * in physical memory):
 */
typedef struct
{
  unsigned char steprate_headunload;
  unsigned char headload_ndma;
  unsigned char motor_delay_off;    /* specified in clock ticks */
  unsigned char bytes_per_sector;
  unsigned char sectors_per_track;
  unsigned char gap_length;
  unsigned char data_length;        /* used only when bytes per sector == 0 */
  unsigned char format_gap_length;
  unsigned char filler;
  unsigned char head_settle_time;   /* specified in milliseconds */
  unsigned char motor_start_time;   /* specified in 1/8 seconds */
} __attribute__ ((packed)) FLOPPY_PARAMS;

FLOPPY_PARAMS floppy_params;

/*
 * Read floppy disk parameters from physical memory into local data
 * structure:
 */
void read_floppy_params()
{
	memcpy((uint8*) &floppy_params, (uint8*) FLOPPY_PARAMS_ADDRESS, sizeof(FLOPPY_PARAMS));
}

void reset_fdc(uint8 base, char drive)
{
	outportb((base + DIGITAL_OUTPUT_REG), 0x00); /* Disable controller */
	outportb((base + DIGITAL_OUTPUT_REG), 0x0c); /* Enable controller */
}


